@@ -6,16 +6,18 @@ import re
 import secrets
 import sys
 import traceback
-import uuid
 from datetime import datetime, timedelta, date
+from typing import Tuple, Union, List, Any, Optional
 from urllib.parse import quote
+from uuid import UUID, uuid4, uuid1
 
 import dateutil.parser
 import pytz
 import xmltodict
 import yaml
 
-from sysnet_pyutils.ident import generate_pid, check_pid, correct_pid, generate_id12
+from sysnet_pyutils.constants import CONFIG_PASSWD
+from sysnet_pyutils.ident import generate_pid, check_pid, correct_pid, generate_id12, PID_PREFIX
 
 TZ = os.getenv('TZ', 'Europe/Prague')
 DEBUG = os.getenv("DEBUG", 'True').lower() in ('true', '1', 't')
@@ -47,6 +49,24 @@ class ConfigFlag(object, metaclass=Singleton):
         logging.info('ConfigFlag constructor')
 
 
+class LoggedObject(object):
+    def __init__(self, object_name):
+        self.name = object_name
+        self._logger = logging.getLogger(object_name)
+
+    @property
+    def log(self):
+        return self._logger
+
+    @log.setter
+    def log(self, logger_ext=None):  # lazy logger
+        if logger_ext is None:
+            logger_ext = logging.getLogger(self.name)
+        if (self._logger is None) or (self._logger.name != logger_ext.name):
+            self._logger = logger_ext
+            self._logger.debug(f"{self.name} logger set to {self._logger.name}")
+
+
 CF = ConfigFlag()
 
 
@@ -107,7 +127,7 @@ class Config(object, metaclass=Singleton):
         self.store()
 
 
-def url_safe(url):
+def url_safe(url: str) -> str:
     """
     Upraví URL, aby neobsahovalo nepovolené znaky
 
@@ -117,7 +137,7 @@ def url_safe(url):
     return quote(url, safe='/:?=&')
 
 
-def who_am_i():
+def who_am_i() -> str:
     """
     Vrátí název funkce
 
@@ -125,10 +145,10 @@ def who_am_i():
     """
     stack = traceback.extract_stack()
     file_name, code_line, func_name, text = stack[-2]
-    return func_name
+    return str(func_name)
 
 
-def unique_list(input_list):
+def unique_list(input_list: list) -> list:
     """
     Vyřadí opakující se položky ze seznamu
 
@@ -144,7 +164,7 @@ def unique_list(input_list):
     return out
 
 
-def api_keys_init(agenda='main', amount=4):
+def api_keys_init(agenda: str='main', amount: int =4) -> List[dict]:
     """
     Vygeneruje klíče pro API
 
@@ -158,7 +178,11 @@ def api_keys_init(agenda='main', amount=4):
     return out
 
 
-def uuid_next(uuid_type=4):
+def uuid_factory():
+    return str(uuid4())
+
+
+def uuid_next(uuid_type: int = 4) -> UUID:
     """
     Vygeneruje UUID
 
@@ -166,13 +190,20 @@ def uuid_next(uuid_type=4):
     :return: uuid
     """
     if uuid_type == 4:
-        out = uuid.uuid4()
+        out = uuid4()
     else:
-        out = uuid.uuid1()
+        out = uuid1()
     return out
 
 
-def pid_next():
+def pid_factory(prefix=None) -> str:
+    p = PID_PREFIX
+    if prefix not in [None, ''] and len(prefix) > 2:
+        p = prefix[:3].upper()
+    return generate_id12(three_char_prefix=p)
+
+
+def pid_next() -> str:
     """
     Vygeneruje korektní PID
 
@@ -181,7 +212,7 @@ def pid_next():
     return generate_pid()
 
 
-def pid_check(pid):
+def pid_check(pid: str) -> bool:
     """
     Zkontroluje korektnost PID
 
@@ -191,7 +222,7 @@ def pid_check(pid):
     return check_pid(pid)
 
 
-def pid_correct(pid):
+def pid_correct(pid: str) -> str:
     """
     Opraví PID
 
@@ -201,7 +232,7 @@ def pid_correct(pid):
     return correct_pid(pid)
 
 
-def id12_next(three_char_prefix=None):
+def id12_next(three_char_prefix: Union[str, None]=None) -> str:
     """
     Vygeneruje korektní 12místný alfanumerický identifikátor s pevným prefixem
 
@@ -211,7 +242,7 @@ def id12_next(three_char_prefix=None):
     return generate_id12(three_char_prefix=three_char_prefix)
 
 
-def api_key_next(name, length=16):
+def api_key_next(name: Union[str, None], length: int = 16) -> dict:
     """
     Vygeneruje slovník API key {<API Key>: <name>}
 
@@ -219,11 +250,13 @@ def api_key_next(name, length=16):
     :param length:  Délka API klíče
     :return:    Slovník {<API Key>: <name>}
     """
+    if name in [None, '']:
+        name = 'main'
     out = {api_key_generate(length=length): name}
     return out
 
 
-def api_key_generate(length: int):
+def api_key_generate(length: int = 16) -> Union[str, None]:
     """
     vygeneruje API klíč
 
@@ -233,7 +266,7 @@ def api_key_generate(length: int):
     return secrets.token_urlsafe(length)
 
 
-def hash_md5(text):
+def hash_md5(text: str) -> Union[str, None]:
     """
     Vytvoří md5 checksum ze zdrojového textu
     :param text:
@@ -241,28 +274,46 @@ def hash_md5(text):
     """
     if text is None:
         return None
-    return hashlib.md5(text.encode("utf-8")).hexdigest()
+    return str(hashlib.md5(text.encode("utf-8")).hexdigest())
 
 
-def hash_sha1(text):
+def hash_sha1(text: str) -> Union[str, None]:
+    """
+    Vytvoří sha1 checksum ze zdrojového textu
+
+    :param text:
+    :return:
+    """
     if text is None:
         return None
-    return hashlib.sha1(text.encode("utf-8")).hexdigest()
+    return str(hashlib.sha1(text.encode("utf-8")).hexdigest())
 
 
-def hash_sha256(text):
+def hash_sha256(text: str) -> Union[str, None]:
+    """
+    Vytvoří sha265 checksum ze zdrojového textu
+
+    :param text:
+    :return:
+    """
     if text is None:
         return None
     return hashlib.sha256(text.encode("utf-8")).hexdigest()
 
 
-def hash_sha384(text):
+def hash_sha384(text: str) -> Union[str, None]:
+    """
+    Vytvoří sha265 checksum ze zdrojového textu
+
+    :param text:
+    :return:
+    """
     if text is None:
         return None
     return hashlib.sha384(text.encode("utf-8")).hexdigest()
 
 
-def is_valid_unid(unid):
+def is_valid_unid(unid: str) -> bool:
     """
     Kontrola validity HCL Notes UNIID
 
@@ -282,7 +333,7 @@ def is_valid_unid(unid):
         return False
 
 
-def is_valid_uuid(value):
+def is_valid_uuid(value: Union[UUID, str]) -> bool:
     """
     Kontrola validity uuid
 
@@ -290,13 +341,13 @@ def is_valid_uuid(value):
     :return:
     """
     try:
-        uuid.UUID(str(value))
+        UUID(str(value))
         return True
     except ValueError:
         return False
 
 
-def is_valid_pid(value):
+def is_valid_pid(value: str) -> bool:
     """
     Kontrola validity PID
     :param value:
@@ -306,7 +357,7 @@ def is_valid_pid(value):
     return bool(re.search(pattern, value))
 
 
-def is_valid_ico(ico):
+def is_valid_ico(ico: str) -> bool:
     """
     Kontrola validity IČO
 
@@ -332,7 +383,20 @@ def is_valid_ico(ico):
     return True
 
 
-def repair_ico(ico):
+def  is_valid_email(email: str) -> bool:
+    """
+    Kontrola validity emailové adresy
+
+    :param email:
+    :return:
+    """
+    if email is None:
+        return False
+    out = bool(re.search(r"^[\w.+\-]+@\w+\.[a-z]{2,3}$", email))
+    return out
+
+
+def repair_ico(ico: str) -> Union[str, None]:
     """
     Opraví IČO
 
@@ -358,7 +422,7 @@ def repair_ico(ico):
     return ico
 
 
-def iso_to_local_datetime(isodate):
+def iso_to_local_datetime(isodate: str) -> Union[datetime, None]:
     """
     ISO string datum do lokálního datetime
 
@@ -373,18 +437,21 @@ def iso_to_local_datetime(isodate):
     return out
 
 
-def convert_hex_to_int(id_hex):
+def convert_hex_to_int(id_hex: str) -> Union[int, None]:
     """
     Konvertuje hex string na int
 
     :param id_hex:  Hexadecimální string
     :return: int
     """
-    id_int = int(id_hex, base=16)
-    return id_int
+    try:
+        id_int = int(id_hex, base=16)
+        return id_int
+    except ValueError:
+        return None
 
 
-def increment_date(date_str=None, days=1):
+def increment_date(date_str: str =None, days: int = 1) -> Union[str, None]:
     """
     Inkrementuje datum v textovém formátu ISO o daný počet dní
 
@@ -395,13 +462,16 @@ def increment_date(date_str=None, days=1):
     if date_str is None:
         return None
     if days is None:
-        return date_str
-    d = datetime.strptime(date_str, '%Y-%m-%d')
-    out = d + timedelta(days=days)
-    return out.date().isoformat()
+        days = 1
+    try:
+        d = datetime.strptime(date_str, '%Y-%m-%d')
+        out = d + timedelta(days=days)
+        return out.date().isoformat()
+    except ValueError:
+        return None
 
 
-def today():
+def today() -> str:
     """
     Vrací ISO 8601 datum dnešního dne
 
@@ -411,7 +481,7 @@ def today():
     return out.date().isoformat()
 
 
-def tomorrow():
+def tomorrow() -> str:
     """
     Vrací ISO 8601 datum zítřejšího dne
 
@@ -420,7 +490,7 @@ def tomorrow():
     return increment_date(date_str=today(), days=1)
 
 
-def cs_bool(value=None):
+def cs_bool(value=None) -> str:
     """
     Vrátí českou textovou hodnotu 'ano'/'ne' pokud je bool(value) True/False
 
@@ -433,7 +503,7 @@ def cs_bool(value=None):
     return out
 
 
-def cron_to_dict(cron):
+def cron_to_dict(cron: str) -> Union[dict, None]:
     """
     Konvertuje cron text do do slovníku
 
@@ -453,10 +523,11 @@ def cron_to_dict(cron):
     return out
 
 
-def date_to_datetime(date_value):
+def date_to_datetime(date_value: Union[date, datetime]) -> Union[datetime, None]:
     """
     Konvertuje date na datetime v lokální časové zóně
-    :param date_value:  hodnota
+
+    :param date_value:  datum/datum a čas
     :return:        hodnota date v lokální časové zóně
     """
     if date_value is None:
@@ -470,12 +541,12 @@ def date_to_datetime(date_value):
     return out
 
 
-def date_to_datetime_utc(date_value):
+def date_to_datetime_utc(date_value: Union[date, datetime]) -> Union[datetime, None]:
     """
     Konvertuje datum na tatum a čas v UTC.
     Vhodné pro MongoDB pro ukládání položek typu datum.
 
-    :param date_value:  hodnota
+    :param date_value:  datum/datum a čas
     :return:    hodnota date v UTC
     """
     if date_value is None:
@@ -490,7 +561,7 @@ def date_to_datetime_utc(date_value):
     return None
 
 
-def date_to_iso(date_value):
+def date_to_iso(date_value: Union[date, datetime]) -> Union[str, None]:
     """
     Konvertuje hodnotu typu date nebo datetime na ISO string
 
@@ -504,7 +575,7 @@ def date_to_iso(date_value):
     return None
 
 
-def remove_empty(source_list):
+def remove_empty(source_list: List[Any]):
     """
     Odstraní prázdné položky ze seznamu
 
@@ -515,13 +586,12 @@ def remove_empty(source_list):
         return None
     target_list = []
     for item in source_list:
-        if item is not None:
-            if bool(item):
-                target_list.append(item)
+        if bool(item):
+            target_list.append(item)
     return target_list
 
 
-def is_base64(body):
+def is_base64(body: Union[str, bytes]) -> bool:
     """
     Kontrola, zda jsou data kódována base64
 
@@ -530,7 +600,7 @@ def is_base64(body):
     """
     try:
         if isinstance(body, str):
-            # If there's any unicode here, an exception will be thrown and the function will return false
+            # If there's any Unicode here, an exception will be thrown and the function will return false
             sb_bytes = bytes(body, 'ascii')
         elif isinstance(body, bytes):
             sb_bytes = body
@@ -542,7 +612,7 @@ def is_base64(body):
         return False
 
 
-def to_base64(body):
+def to_base64(body: Union[str, bytes]) -> Union[bytes, None]:
     """
     Zajistí, aby data byla v base64
 
@@ -553,10 +623,10 @@ def to_base64(body):
         return body
     try:
         if isinstance(body, str):
-            # If there's any unicode here, an exception will be thrown and the function will return false
-            sb_bytes = bytes(body, 'ascii')
+            # If there's any Unicode here, an exception will be thrown and the function will return false
+            sb_bytes: bytes = bytes(body, 'ascii')
         elif isinstance(body, bytes):
-            sb_bytes = body
+            sb_bytes: bytes = body
         else:
             raise ValueError("Argument must be string or bytes")
         return base64.b64encode(sb_bytes)
@@ -565,7 +635,7 @@ def to_base64(body):
         return None
 
 
-def encode_string_b64(data: str, encoding='utf-8'):
+def encode_string_b64(data: str, encoding='utf-8') -> Union[str, None]:
     """
     Zakóduje string do base64
 
@@ -579,7 +649,7 @@ def encode_string_b64(data: str, encoding='utf-8'):
     return out
 
 
-def decode_b64_string(b64_data: str, encoding='utf-8'):
+def decode_b64_string(b64_data: str, encoding='utf-8') -> Union[str, None]:
     """
     Dekóduje base64 data na string
 
@@ -607,7 +677,7 @@ def encode_file_b64(filepath, encoding='utf-8'):
     return out
 
 
-def decode_b64_to_file(b64_data: str, filepath, encoding='utf-8'):
+def decode_b64_to_file(b64_data: str, filepath: str, encoding='utf-8') -> Union[str, None]:
     """
     Uloží base64 data do souboru
 
@@ -664,13 +734,13 @@ class Log(object, metaclass=Singleton):
             self.logger = ext_logger
 
 
-def to_camel(s):
+def to_camel(s: str) -> str:
     temp = re.split('_+', s)
     out = temp[0] + ''.join(map(lambda x: x.title(), temp[1:]))
     return out
 
 
-def to_snake(s):
+def to_snake(s: str) -> str:
     # return re.sub('([A-Z]\w+$)', '_\\1', s).lower()
     return re.sub(r'([A-Z]\w+$)', '_\\1', s).lower()
 
@@ -687,7 +757,7 @@ def to_camel_dict(d):
     return {to_camel(a): to_camel_dict(b) if isinstance(b, (dict, list)) else b for a, b in d.items()}
 
 
-def xml_to_dict(xml_text):
+def xml_to_dict(xml_text: str) -> Union[dict, None]:
     """
     Parsuje XML string do XML dictionary
     :param xml_text:    XML text
@@ -696,7 +766,7 @@ def xml_to_dict(xml_text):
     return xmltodict.parse(xml_text)
 
 
-def dict_to_xml(xml_dict):
+def dict_to_xml(xml_dict: dict) -> Union[str, None]:
     """
     Parsuje XML dict do XML textu
 
@@ -712,11 +782,11 @@ ORDER_BASE = 26
 CHAR_BASE = 65
 
 
-def order_to_cites(order: int):
+def order_to_cites(order: int) -> str:
     """
     Konvertuje celočíselnou hodnotu na písmennou
 
-    :param order: celočíselná hodnota např. 1458
+    :param order: Celočíselná hodnota např. 1458
     :return:    znaková hodnota např. 'BDB'
     """
     if order is None:
@@ -742,11 +812,11 @@ def order_to_cites(order: int):
     return out
 
 
-def cites_to_order(cites: str):
+def cites_to_order(cites: str) -> int:
     """
     Konvertuje písmennou hodnotu na celočíselnou
     
-    :param cites:   znaková hodnota např. 'BDB' 
+    :param cites:   Znaková hodnota např. 'BDB'
     :return:    celočíselná hodnota např. 1458
     """
     if cites is None:
@@ -766,43 +836,48 @@ def cites_to_order(cites: str):
     return out
 
 
-def local_now():
+def local_now() -> datetime:
     """
     Vrací aktuální časovou značku v lokální časové zóně.
 
     :return:
     """
-    return datetime_now(tz=TZ)
+    # return datetime_now(tz=TZ)
+    return datetime.now(tz=pytz.timezone(TZ))
 
 
-def gmt_now():
+def gmt_now() -> datetime:
     """
     Vrací aktuální časovou značku ve světovém čase.
 
     :return:
     """
-    return datetime_now(tz='GMT')
+    # return datetime_now(tz='GMT')
+    return datetime.now(tz=pytz.utc)
 
 
-def datetime_now(tz='Europe/Prague'):
+def datetime_now(tz='Europe/Prague') -> datetime:
     """
     Vrací aktuální časovou značku ve vybrané časové zóně.
 
     :return:
     """
-    local_tz = pytz.timezone(tz)
-    return datetime.now(tz=local_tz)
+    return datetime.now(tz=pytz.timezone(tz))
 
 
-def timestamp():
+def timestamp(local=False) -> str:
     """
     Vrací textovou podobu časové značky. Vhodné pro názvy souborů.
+
+    :param local: Lokální časová zóna
     """
     out = gmt_now()
+    if local:
+        out = local_now()
     return out.strftime('%Y%m%d%H%M%S')
 
 
-def local_datetime(value=None):
+def local_datetime(value=None) -> Optional[datetime]:
     """
     Vrací datum a čas v lokální
 
@@ -814,3 +889,97 @@ def local_datetime(value=None):
         return value
     local_tz = pytz.timezone(TZ)
     return value.astimezone(local_tz)
+
+
+def parse_ldap_name(ldap_name: str) -> Tuple[Union[str, None], Union[str, None]]:
+    """
+    Parsuje LDAP jméno do CN.
+
+    :param ldap_name:
+    :return:    Tuple CN a původní jméno.
+    """
+    if ldap_name in [None, '']:
+        return None, None
+    if 'cn=' not in ldap_name.lower():
+        return ldap_name, None
+    if ',' in ldap_name:
+        ldap = ldap_name.split(',')
+    elif ', ' in ldap_name:
+        ldap = ldap_name.split(', ')
+    elif '/' in ldap_name:
+        ldap = ldap_name.split('/')
+    else:
+        ldap = [ldap_name]
+    cn = None
+    for item in ldap:
+        if 'cn=' in item:
+            cn = item.split('=')[-1]
+            break
+        if 'CN=' in item:
+            cn = item.split('=')[-1]
+            break
+    return cn, ldap_name
+
+
+class Context(LoggedObject, metaclass=Singleton):
+    def __init__(
+            self,
+            config=None,
+            api_key=None,
+            user_name=None,
+            agenda=None,
+            cert_file=None,
+            cert_pass=None):
+        super().__init__(object_name='CONTEXT')
+        self.config = config
+        if self.config is None:
+            self.log.error(f"{self.name} FAILED - Missing config")
+        self.api_key = api_key
+        self.user_name = user_name
+        self.agenda = agenda
+        self.authenticated = False
+        self.cert_file = cert_file
+        self.cert_pass = cert_pass
+        self.log.info(f"{self.name} Created")
+
+    def clear(self):
+        self.api_key = None
+        self.user_name = None
+        self.agenda = None
+        self.authenticated = False
+        self.cert_file = None
+        self.cert_pass = None
+        self.log.info(f"{self.name} Cleared")
+
+    def check_api_key(self, api_key):
+        self.clear()
+        if api_key in [None, '']:
+            return False
+        for agenda in self.config.keys():
+            if 'api_keys' not in self.config[agenda]:
+                continue
+            for ak in self.config[agenda]['api_keys']:
+                if api_key in ak.keys():
+                    self.api_key = api_key
+                    self.agenda = agenda
+                    self.user_name = ak[api_key]
+                    self.authenticated = True
+                    self.log.info(f"{self.name} - User '{self.user_name}' logged in")
+                    break
+            if self.authenticated:
+                break
+        return self.authenticated
+
+    def check_user(self, username, password):
+        self.clear()
+        for agenda in self.config.keys():
+            if CONFIG_PASSWD not in self.config[agenda]:
+                continue
+            if username in self.config[agenda][CONFIG_PASSWD]:
+                if self.config[agenda][CONFIG_PASSWD][username] == password:
+                    self.api_key = None
+                    self.agenda = agenda
+                    self.user_name = username
+                    self.authenticated = True
+                    break
+        return self.authenticated
